@@ -22,10 +22,12 @@ import json
 
 from angus import rest
 import angus
+import six
 
-__updated__ = "2015-06-07"
+
+__updated__ = "2016-11-30"
 __author__ = "Aurélien Moreau"
-__copyright__ = "Copyright 2015, Angus.ai"
+__copyright__ = "Copyright 2015-2016, Angus.ai"
 __credits__ = ["Aurélien Moreau", "Gwennael Gate"]
 __license__ = "Apache v2.0"
 __maintainer__ = "Aurélien Moreau"
@@ -59,6 +61,7 @@ class CompositeService(rest.Resource):
         super(CompositeService, self).__init__(*args, **kwargs)
         self.root = Root(conf=self.conf)
         self.default_session = None
+        self.session_parameters = None
 
     def process(self, parameters, async=False, session=None, callback=None):
         if parameters is None:
@@ -66,7 +69,10 @@ class CompositeService(rest.Resource):
         else:
             parameters = copy.copy(parameters)
 
-        if session is not None:
+        if self.session_parameters is not None:
+            parameters.update(self.session_parameters)
+
+        if session is None:
             session = self.default_session
 
         if session is not None:
@@ -79,7 +85,7 @@ class CompositeService(rest.Resource):
         data = json.dumps(parameters, cls=generate_encoder(self.root))
 
         futures = []
-        for service in self.services:
+        for name, service in six.iteritems(self.services):
             if attachments:
                 files = attachments + \
                     [('meta', (None, data, 'application/json'))]
@@ -90,7 +96,7 @@ class CompositeService(rest.Resource):
                     service.jobs.endpoint,
                     data=data,
                     headers=headers)
-            futures.append((service.name, r))
+            futures.append((name, r))
 
         result = {}
         for (name, r) in futures:
@@ -98,10 +104,7 @@ class CompositeService(rest.Resource):
             if r.status_code < 400:
                 result[name] = r.json()
 
-        result = {
-            "status": 200,
-            "composite": result,
-        }
+        result["status"] = 200
 
         job = rest.Job(
             self.endpoint, "", representation=result, conf=self.conf)
@@ -115,10 +118,17 @@ class CompositeService(rest.Resource):
         session = rest.Session(self)
         return session
 
-    def enable_session(self):
+    def enable_session(self, parameters=None):
         if self.default_session is None:
             self.default_session = self.create_session()
 
+        if parameters is None:
+            parameters = {}
+        else:
+            parameters = copy.copy(parameters)
+
+        self.session_parameters = parameters
+
     def disable_session(self):
         self.default_session = None
 
@@ -146,7 +156,7 @@ class ServiceDirectory(rest.Collection):
         if services is None:
             services = description
 
-        cservices = []
+        cservices = dict()
         for name in services:
             if isinstance(name, tuple):
                 version = name[1]
@@ -155,7 +165,7 @@ class ServiceDirectory(rest.Collection):
                 version = None
 
             service = self.get_service(name, version)
-            cservices.append(service)
+            cservices[name] = service
 
         return CompositeService(
             "memory:///", "composite", services=cservices, conf=self.conf)
