@@ -25,16 +25,17 @@ import uuid
 from six.moves.urllib import parse as urlparse
 import requests
 import requests_futures.sessions
+import logging
 
-
-__updated__ = "2015-06-08"
+__updated__ = "2016-11-30"
 __author__ = "Aurélien Moreau"
-__copyright__ = "Copyright 2015, Angus.ai"
+__copyright__ = "Copyright 2015-2016, Angus.ai"
 __credits__ = ["Aurélien Moreau", "Gwennael Gate"]
 __license__ = "Apache v2.0"
 __maintainer__ = "Aurélien Moreau"
 __status__ = "Production"
 
+logger = logging.getLogger('AngusSDK')
 
 class Configuration(requests_futures.sessions.FuturesSession):
 
@@ -121,6 +122,37 @@ class Collection(Resource):
         return resource_type(
             self.endpoint, result['url'], representation=result, conf=self.conf)
 
+
+    def create_async(self, parameters, callback, resource_type=Resource):
+
+        def store(res, resource_type, callback):
+            res = res.result()
+            res.raise_for_status()
+
+            result = res.json()
+            callback(resource_type(self.endpoint, result['url'],
+                             representation=result, conf=self.conf))
+
+        attachments = []
+
+        data = json.dumps(parameters, cls=generate_encoder(attachments))
+
+        if self.conf.executor._work_queue.qsize() > self.conf.executor._max_workers:
+            logger.warn("There are too many requests awaiting to "
+            "be sent. This request will be ignored. Please try to decrease "
+            "the rate at which \"process\" is called.")
+
+
+        if attachments:
+            files = attachments + [('meta', (None, data, 'application/json'))]
+            r = self.conf.post(self.endpoint, files=files)
+        else:
+            headers = {'content-type': 'application/json'}
+            r = self.conf.post(self.endpoint, data=data, headers=headers)
+
+        r.add_done_callback(lambda fut: store(fut, resource_type, callback))
+        return None
+
     def list(self, filters):
         r = self.conf.get(self.endpoint, params=filters)
 
@@ -160,6 +192,7 @@ class Service(Resource):
         self.jobs = Collection(self.endpoint, 'jobs',
                                conf=self.conf)
         self.default_session = None
+        self.session_parameters = None
 
     def process(
             self, parameters=None, async=False, session=None, callback=None):
@@ -168,6 +201,9 @@ class Service(Resource):
         else:
             parameters = copy.copy(parameters)
 
+        if self.session_parameters is not None:
+            parameters.update(self.session_parameters)
+
         if session is None:
             session = self.default_session
 
@@ -176,14 +212,17 @@ class Service(Resource):
 
         parameters['async'] = async
 
-        job = self.jobs.create(
-            parameters,
-            resource_type=Job)
+        if callback:
+            job = self.jobs.create_async(
+                parameters,
+                callback,
+                resource_type=Job)
+        else:
+            job = self.jobs.create(
+                parameters,
+                resource_type=Job)
+            return job
 
-        if job.status == Resource.CREATED and callback:
-            callback(job)
-
-        return job
 
     def get_description(self):
         return self.description.fetch()
@@ -192,10 +231,17 @@ class Service(Resource):
         session = Session(self)
         return session
 
-    def enable_session(self):
+    def enable_session(self, parameters=None):
         if self.default_session is None:
             self.default_session = self.create_session()
 
+        if parameters is None:
+            parameters = {}
+        else:
+            parameters = copy.copy(parameters)
+
+        self.session_parameters = parameters
+
     def disable_session(self):
         self.default_session = None
 
